@@ -14,17 +14,15 @@ typedef Eigen::MappedSparseMatrix<double> MSpMat;
 typedef Eigen::SparseMatrix<double> SpMat;
 typedef Eigen::SimplicialLDLT<SpMat> SpChol;
 
-// simple example of creating two matrices and
-// returning the result of an operatioon on them
 //
 // via the exports attribute we tell Rcpp to make this function
 // available from R
 //
 // [[Rcpp::export]]
-Rcpp::List invsph(Eigen::SparseMatrix<double> & A, const Eigen::Map<Eigen::VectorXd> der, const Eigen::Map<Eigen::VectorXd> dv,
-                  const Eigen::Map<Eigen::VectorXd> v1, const Eigen::MatrixXd & mx, const Eigen::Map<Eigen::VectorXd> v2, 
-                  const Eigen::Map<Eigen::VectorXd> v3,const Eigen::MatrixXd & v4,
-                  const Eigen::Map<Eigen::VectorXd> av, const Eigen::Map<Eigen::VectorXd> bw, 
+Rcpp::List invsph(Eigen::SparseMatrix<double> & A, const Eigen::Map<Eigen::VectorXd> & der, const Eigen::Map<Eigen::VectorXd> & dv,
+                  const Eigen::Map<Eigen::VectorXd> & v1, const Eigen::Map<Eigen::MatrixXd> & mx, const Eigen::Map<Eigen::VectorXd> & v2, 
+                  const Eigen::Map<Eigen::VectorXd> & v3,const Eigen::Map<Eigen::MatrixXd> & v4,
+                  const Eigen::Map<Eigen::VectorXd> & av, const Eigen::Map<Eigen::VectorXd> & bw, 
                   const Eigen::VectorXd & f,const Eigen::VectorXd & inv,const Eigen::VectorXd & tau,
                   const int sol) {
   
@@ -118,7 +116,8 @@ Rcpp::List invsph(Eigen::SparseMatrix<double> & A, const Eigen::Map<Eigen::Vecto
     for(int k=0; k<nc_1; k++)
       wb_sig_i_hx_der.col(k) = sd_v*wb_sig_i_hx_der.col(k).array();
   }else{
-    wb_sig_i_hx_der = tau_i*wb_sig_i_hx_der;
+    if(tau_i!=1)
+    {wb_sig_i_hx_der = tau_i*wb_sig_i_hx_der;}
   }
   
   if(f(0)==0)
@@ -166,7 +165,8 @@ Rcpp::List invsph(Eigen::SparseMatrix<double> & A, const Eigen::Map<Eigen::Vecto
         for(int k=0; k<nc_1; k++)
           wb_sig_i_hx_der2.col(k) = sd_v*wb_sig_i_hx_der2.col(k).array();
       }else{
-        wb_sig_i_hx_der2 = tau_i*wb_sig_i_hx_der2;
+        if(tau_i!=1)
+        {wb_sig_i_hx_der2 = tau_i*wb_sig_i_hx_der2;}
       }
       wb_sig_i_hx_der = wb_sig_i_hx_der+wb_sig_i_hx_der2;
     }
